@@ -8,6 +8,7 @@ import os
 import pathlib
 import platform
 import re
+import shutil
 import sys
 import sysconfig
 from typing import Any, Sequence, cast
@@ -20,6 +21,7 @@ from lsprotocol.types import (
     TEXT_DOCUMENT_DID_CLOSE,
     TEXT_DOCUMENT_DID_OPEN,
     TEXT_DOCUMENT_DID_SAVE,
+    TEXT_DOCUMENT_FORMATTING,
     TEXT_DOCUMENT_HOVER,
     AnnotatedTextEdit,
     ClientCapabilities,
@@ -34,6 +36,7 @@ from lsprotocol.types import (
     DidCloseTextDocumentParams,
     DidOpenTextDocumentParams,
     DidSaveTextDocumentParams,
+    DocumentFormattingParams,
     Hover,
     HoverParams,
     InitializeParams,
@@ -451,6 +454,21 @@ def resolve_code_action(params: CodeAction) -> CodeAction:
     return params
 
 
+@LSP_SERVER.feature(TEXT_DOCUMENT_FORMATTING)
+def formatting(params: DocumentFormattingParams) -> list[TextEdit] | None:
+    """LSP handler for textDocument/formatting request."""
+    document = LSP_SERVER.workspace.get_document(params.text_document.uri)
+
+    # Deep copy, to prevent accidentally updating global settings.
+    settings = copy.deepcopy(_get_settings_by_document(document))
+
+    if settings["fixAll"]:
+        return _formatting_helper(document)
+    elif settings["organizeImports"]:
+        return _formatting_helper(document, only="I001")
+    return None
+
+
 @LSP_SERVER.command("ruff.applyAutofix")
 def apply_autofix(arguments: tuple[TextDocument]):
     uri = arguments[0]["uri"]
@@ -748,7 +766,11 @@ def _executable_path(settings: dict[str, Any]) -> str:
     # If the interpreter is same as the interpreter running this process, get the
     # script path directly.
     path = os.path.join(sysconfig.get_path("scripts"), TOOL_MODULE)
-    if bundle and not os.path.exists(path):
+    environment_path = shutil.which("ruff")
+    if environment_path:
+        log_to_output(f"Using environment executable: {environment_path}")
+        path = environment_path
+    elif bundle and not os.path.exists(path):
         log_to_output(
             f"Interpreter executable ({path}) not found; "
             f"falling back to bundled executable: {bundle}"
